# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2017
# Author: Benjamin Berg <bberg@redhat.com>

class Event(object):
    pass

class ReadyEvent(Event):
    pass

class SubtestDoneEvent(Event):
    def __init__(self, subtest, **kwargs):
        self.subtest = subtest
        self.result = kwargs

class WarningEvent(Event):
    def __init__(self, warning):
        self.warning = warning

class FailEvent(Event):
    def __init__(self, fail, no_user_confirm=False):
        self.fail = fail
        self.no_user_confirm = no_user_confirm

class CancelEvent(Event):
    def __init__(self, cancel, no_user_confirm=False):
        self.cancel = cancel
        self.no_user_confirm = no_user_confirm

class InfoEvent(Event):
    def __init__(self, info):
        self.info = info

class DebugEvent(Event):
    def __init__(self, debug):
        self.debug = debug

class Signal(object):
    pass

class SubtestSignal(object):
    def __init__(self, subtest, **kwargs):
        self.subtest = subtest
        self.args = kwargs

class StateSignal(object):
    def __init__(self, state):
        self.state = state

class QuitSignal(Signal):
    pass

class TimeoutError(Exception):
    pass

class UIError(Exception):
    pass

class UIDoneError(UIError):
    pass




class TestUI(object):
    def __init__(self, app):
        self.app = app

    def recv(self, event):
        pass

    def get_widget(self):
        return None

class TestUICtrl(object):

    def __init__(self, app_ctrl):
        self.app_ctrl = app_ctrl

    def recv(self, signal):
        pass


class UIRegister(type):
    _register = []

    def __new__(*args, **kwargs):
        cls = type.__new__(*args, **kwargs)
        UIRegister._register.append(cls)

class UICtrlRegister(type):
    _register = []

    def __new__(*args, **kwargs):
        cls = type.__new__(*args, **kwargs)
        UICtrlRegister._register.append(cls)


