# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Original code by Nick Dekker and Marthe Veldhuis was licensed under MIT.
#
# Copyright: 2017 Nick Dekker, Marthe Veldhuis.
# Copyright: Red Hat Inc. 2017
# Author: Benjamin Berg <bberg@redhat.com>

import subprocess
import gi
gi.require_version('NM', '1.0')
from gi.repository import GLib, NM, GObject


def ping(host, interface, timeout=2):
    """
    pingtest checks whether the IP is reachable on the given
    interface using a single ICMP ping.

    :param host: The host to ping
    :param interface: The interface to use as the origin for the ping
    :param timeout: Timeout for the ping command
    :return: True on success
    
    """
    response = subprocess.call(['ping', '-I', interface, host,
                                        '-c', '1',
                                        '-W', str(timeout)])

    if response == 0:
        return True
    return False

def nm_get_device(device_path):
    # .new is the sync routine, otherwise GObject.__init__ is called
    client = NM.Client.new(None)
    return client.get_device_by_path(device_path)

def nm_get_devices(device_types=None):
    # .new is the sync routine, otherwise GObject.__init__ is called
    client = NM.Client.new(None)
    res = list()
    for device in client.get_devices():
        if device_types is not None and device.props.device_type not in device_types:
            continue
        res.append(device)

    return res

def get_known():
    """
    Finds a list of known (existing) networks connections in the system
    
    :return: The list of known networks
    
    """
    # .new is the sync routine, otherwise GObject.__init__ is called
    client = NM.Client.new(None);
    connections = client.get_connections()
    return connections

if __name__ == '__main__':
    for d in nm_get_devices([NM.DeviceType.WIFI]):
      print d.props.product
