#!/usr/bin/env python2
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2017
# Author: Benjamin Berg <bberg@redhat.com>

import os
import os.path
import re
import evdev
import select

from avocado import main
from avocado.core import exceptions

from fed_laptoptest.test import SessionTest, interactive
from fed_laptoptest.hwinfo import HWinfo
from fed_laptoptest.ui import Signal, Event, TestUI

from gi.repository import Gtk, GLib

import time

class Backlight(SessionTest):
    """
    .. title:: Backlight Control (via GSD)

    This test will change programatically change the brightness of your laptop
    screen a few times to confirm that the controls are working. The test uses
    the GNOME Settings Daemon.

    1. .. class:: action-wait-user

       Waiting for test to start.

    2. .. class:: action-change

       The brightness will move from highest to lowest brightness and back
       again in a short period of time.

    3. .. class:: action-finish

       Please confirm that you saw the brightness change.

    :avocado: enable
    :avocado: tags=manual
    :categories: screen
    """

    def set_backlight(self, percent, timeout=1, retry=0):
        reply = self.app_ctrl.dbus.call_wait(
            'org.gnome.SettingsDaemon.Power',
            '/org/gnome/SettingsDaemon/Power',
            'org.freedesktop.DBus.Properties',
            'Set', GLib.Variant('(ssv)', (
                'org.gnome.SettingsDaemon.Power.Screen',
                'Brightness', GLib.Variant('i', percent)
            )), timeout=timeout, retry=retry
        )

    @interactive
    def test(self):
        time.sleep(0.5)

        try:
            self.set_backlight(100, timeout=5, retry=5)
        except:
            import traceback
            self.log.warn('Error calling GNOME backlight control:')
            self.log.warn(traceback.format_exc())

            self.app_ctrl.prompt.ok_or_fail(
                extra='<b>There appears to be no controllable backlight. ' + \
                      'Please mark the test as success if you can confirm ' + \
                      'that there really is no backlight. Otherwise mark ' + \
                      'it as failure.</b>')

        self.app_ctrl.status.mark_action('wait-user')
        self.app_ctrl.prompt.wait()

        self.app_ctrl.status.mark_action('change')
        for i in xrange(0, 201, 5):
            self.set_backlight(abs(100 - i))
            time.sleep(0.05)
            # Allow user to cancel/fail the test
            self.app_ctrl.read_pending()

if __name__ == "__main__":
    main()

