#!/usr/bin/env python2
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2017
# Author: Benjamin Berg <bberg@redhat.com>

from avocado import main
from avocado.core import exceptions
from fed_laptoptest import chameleon

# Do not import Gtk or similar before importing the test class
from fed_laptoptest import ui
from fed_laptoptest.test import ChameleonTest, batch
from fed_laptoptest.edid import Edid

import time

class PlugStress(ChameleonTest):
    """
    .. title:: Monitor Plugging Stresstest

    This test uses the chameleon board to test plugging and unplugging monitors
    a lot. The goal here is to hit potential race condition in kernel- or
    userspace.

    The frequency of the plug/unplug events is decreased further and further
    until the DUT detects the plug events in userspace.

    :avocado: enable
    :avocado: tags=chameleon,standalone

    :param repeat: The number of times to repeat the test
    :param stable_detections: The number of correct add/remove detections to
        stop testing. i.e. the time the monitor is plugged/unplugged is
        increased until the userspace detects the plug events correctly.
    :categories: graphics
    """

    @batch
    def test(self):
        """
        The test.
        """
        repeat = self.params.get('repeat', default=1)
        required_stable_detections = self.params.get('stable_detections', default=3)

        for port in self.chameleon.iter_ports({'DP', 'HDMI', 'VGA'}):
            self.log.info("Testing port %i (%s)", port, self.chameleon.get_port_type(port))

            for r in xrange(repeat):
                # Don't choose a too low value as otherwise the kernel assumes
                # there is an HDP interrupt storm.
                # NOTE: Maybe it would be smarter to start with a high value
                #       and then go down to e.g. half the rate that allows
                #       successfull detection.
                # 0.5 sleeptime with an increase of a * 1.01 crashes reproducably
                sleeptime = 0.5
                stable_detections = 0

                while stable_detections < required_stable_detections:
                    self.chameleon.plug(port)
                    try:
                        self.app_ctrl.monitor.wait_added(timeout=sleeptime)
                        stable_detections += 1
                        self.log.info('GOOD Detection (count: %i)' % stable_detections)
                    except ui.TimeoutError:
                        stable_detections = 0
                        if sleeptime > 5:
                            raise AssertionError('No detection at all with sleep time %f!' % sleeptime)
                    finally:
                        self.chameleon.unplug(port)
                        time.sleep(sleeptime)
                        sleeptime += sleeptime / 10

            # Ensure unplugged
            self.chameleon.unplug(port)
            self.app_ctrl.monitor.wait_no_external()

if __name__ == "__main__":
    main()

