#!/usr/bin/env python2
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2017
# Author: Benjamin Berg <bberg@redhat.com>

import os
import os.path
import re
import evdev
import select
import time

from avocado import main
from avocado.core import exceptions

from fed_laptoptest.test import SessionTest, interactive
from fed_laptoptest.ui import Event, TestUI
from fed_laptoptest.utils import drm, ui as uiutils

from gi.repository import Gtk, GLib

class SimpleDRMPlugUI(TestUI):

    def update_info(self, output):
        row = output.row
        hbox = row.get_child()
        if not hbox:
            hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)

            hbox.img = Gtk.Image()
            hbox.pack_start(hbox.img, fill=True, expand=False, padding=5)

            hbox.label = Gtk.Label(xalign=0)
            hbox.pack_start(hbox.label, fill=True, expand=True, padding=5)
            row.add(hbox)

        if not hasattr(output, 'was_connected'):
            output.was_connected = output.connected

        if output.connected:
            output.was_connected = True

        hbox.img.set_from_gicon(uiutils.status_to_icon('PASS' if output.was_connected else 'FAIL'), Gtk.IconSize.LARGE_TOOLBAR)

        if not output.connected:
            monitor = 'Disconnected'
        else:
            if output.monitor:
                monitor = output.monitor
            elif output.name.startswith('eDP'):
                monitor = 'Internal Monitor'
            else:
                monitor = 'Unknown Monitor'

            self.app.info('Monitor "{:s}" connected on output {:s} of card {:s} (driver {:s})'.format(
                              monitor, output.name, output.card.descr, output.card.driver))

        hbox.label.set_markup(
            '<b>{output:s} on card {card:s} ({driver:s})</b>\n'
            'Monitor: {monitor:s}'.format(
                card=output.card.descr,
                output=output.name,
                monitor=monitor,
                driver=output.card.driver,
            ))

        row.show_all()

    def _output_changed(self, monitor, output):
        if not hasattr(output, 'row'):
            output.row = Gtk.ListBoxRow()
            self.listbox.add(output.row)
        self.update_info(output)

    def get_widget(self):
        scroll = Gtk.ScrolledWindow()
        self.listbox = Gtk.ListBox()

        scroll.add(self.listbox)
        scroll.show_all()

        self._monitor = drm.DRMMonitor()
        self._monitor.connect('output-changed', self._output_changed)

        return scroll


class SimpleDRMPlugTest(SessionTest):
    """
    .. title:: Basic Monitor Output Testing

    This test will show the monitors currently detected by the system. Right now
    it does *not* yet check whether GNOME has also configured the monitor
    correctly and uses it for output.

    Please note that multiple outputs may be listed for a single port (e.g.
    a DisplayPort output may have multiple DP or HDMI connections). Some outputs
    may only be accessible through a docking station or special adapters.

    This is a simple user driven test. Please only mark it as a failure if you
    had an issue with a display not being detected.

    1. .. class:: action-wait-user

       Any monitor that you plug in should show up on the right side. Please
       mark the test as failure if a monitor is not detected for some reason.
       Feel free to *Cancel* it if you cannot sufficiently test it at this time.

    :avocado: enable
    :avocado: tags=manual
    :categories: graphics
    """

    ui_class = SimpleDRMPlugUI

    def test(self):
        context = GLib.main_context_default()

        # State is purely decided by the user
        self.app_ctrl.status.mark_action('wait-user')
        self.app_ctrl.status.wait_finished()


if __name__ == "__main__":
    main()

